///////////////////////////////////////////////////////
//                                                   //
//   Dev. by Jonathan Biteau (biteau@in2p3.fr)       //
//                       last edit: 2019-09-10       //
//                                                   //
///////////////////////////////////////////////////////

/* This code is provided by the Pierre Auger Collaboration as it is without any warranty, 
for questions please contact Jonathan Biteau [biteau(at)in2p3.fr]. 
If this material from Pierre Auger Collaboration was helpful for your research work, 
we kindly request the following references to be cited in any published material: 
2018ApJ...853L..29A, 2019ICRC...36..206C.*/

// C/C++ classics
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <iomanip> 
#include <sys/stat.h>
#include <numeric>
#include <time.h>
#include <utility>

// ROOT
#include "TRint.h"
#include "TROOT.h"
#include "TStyle.h"
#include "TFile.h"
#include "TGraph.h"
#include "TObjArray.h"
#include "TSystem.h"

#ifdef gcc323
char* operator+( streampos&, char* );
#endif

using namespace std;
void Usage(string sinput)
{
	cout << endl;
	cout << " Synopsis : " << endl;
	cout << sinput << " -h, --help to obtain this message" << endl;
	cout << sinput << " < threshold > " << endl << endl;

	cout << " Description :" << endl;
	cout << " Load the attenuated flux weights, sotred in CompoModel/XXX"<<endl
			 << " -- that's all folks! -- "
			 << endl<< endl;

	exit(0);
}

// Scenario structure used for composition
struct Scenario {
  Scenario(const string& name,
           const unsigned int modelIndex,
           const double fP, const double fHe, const double fN,
           const double fSi, const double fFe) :
    fName(name), fModel(modelIndex) {

    fFraction[0] = fP;
    fFraction[1] = fHe;
    fFraction[2] = fN;
    fFraction[3] = fSi;
    fFraction[4] = fFe;

  }

  static const unsigned int gnMass = 5;
  std::string fName;
  unsigned int fModel;
  double fFraction[gnMass];
};

//Load a weight
double LoadWeight(TFile* crpFile, double distance, const unsigned int* masses, int threshold, const Scenario& s){
	double sum = 0;
	for(unsigned int iMass = 0; iMass < Scenario::gnMass; ++iMass) {
		ostringstream gname;
		gname << "atten" << masses[iMass] << "_" << threshold - 20 << "_" << s.fModel << "_0";
		TGraph* graph = (TGraph*) crpFile->Get(gname.str().c_str());
		if (!graph) {
			cerr << gname.str() << " does not exist " << endl;
			sum = 0;
			break;
		}
		if (distance < *(graph->GetX() + graph->GetN() - 1)) {
			if (distance <= 0) sum += s.fFraction[iMass];
			else sum += s.fFraction[iMass] * graph->Eval(distance);
		}
	}
	return sum;
}


//Load the *burst or AGN model
void LoadModel(string filename, vector< string >& vSrcName, vector< double >& vl, vector< double >& vb, vector< double >& vw, vector< double >& vdist, bool verbose=true)
{
	//Following numbers depend on the format of the file!!
	unsigned int nentries_per_line = 5;

	//Clear the vectors to file
	vl.clear();
	vb.clear();
	vw.clear();
	vdist.clear();

	//Load data from file
	ifstream myfile;	
	myfile.open(filename.c_str());
	if(myfile.is_open()){
		//loops on the lines
		while(myfile.good()){
			string line;
			getline (myfile,line);
			stringstream ss(line);
			string srcName;
			ss>>srcName;
			vector< double > vbuf;
			double buf;;
			while(ss>>buf) vbuf.push_back(buf);
			if(vbuf.size()==nentries_per_line-1){
				vSrcName.push_back(srcName);
				vl.push_back(vbuf[0]);
				vb.push_back(vbuf[1]);
				vw.push_back(vbuf[2]);
				vdist.push_back(vbuf[3]);
			}
			else if(verbose && line.size()>0) cout<<"Line unread in source file: "<<line<<endl;
		}
		myfile.close();
	}
	else cout<<"Could not open file: "<<filename<<endl;
}


/// Main //////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                            Initialization                              //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////

	// Command line
	if(argc != 2) Usage(argv[0]);
	unsigned int threshold = abs(int(atof(argv[1])));

	// Initialize root
	int fargc = 1;
	TRint *rint = NULL;
	rint = new TRint("Test plot", &fargc, argv);
	gROOT->SetStyle("Plain");
	gStyle->SetTitleFont(30,"TITLE");

	// Load the file containing the weights
	const string crpFilename("CompoModel/anaCRP3.root");
	TFile* crpFile = TFile::Open(crpFilename.c_str());
	if (!crpFile) {
		cerr << " error opening " << crpFilename << endl;
		gSystem->Exit(0);
	}

	//Load the models and styles
	static const unsigned int masses[Scenario::gnMass] = {1, 4, 14, 28, 56};
	vector<Scenario> vscenarios;
	vscenarios.push_back(Scenario("EPO1st", 0, 0.000, 0.673, 0.281, 0.046, 0));
	vscenarios.push_back(Scenario("EPO2nd", 1, 0.000, 0.000, 0.798, 0.202, 0));
	vscenarios.push_back(Scenario("Sib1st", 2, 0.702, 0.295, 0.003, 0.000, 0));

	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                         Load the source model                          //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////

	vector< string > vsmodelfile, vssources, vsfile;


	vsmodelfile.push_back("Catalog/sb_sample.dat");
	vssources.push_back("Starburst galaxies");
	vsfile.push_back("sb");

	vsmodelfile.push_back("Catalog/swift_sample.dat");
	vssources.push_back("Swift extragalactic sources");
	vsfile.push_back("swift");

	vsmodelfile.push_back("Catalog/agn_sample.dat");
	vssources.push_back("Active galactic nuclei");
	vsfile.push_back("agn");

	vsmodelfile.push_back("Catalog/2mrs_out_checked _woLG.dat");
	vssources.push_back("2MRS - Local Group");
	vsfile.push_back("2mrs_woLG");

	for(unsigned im=0; im<vsmodelfile.size(); im++){
		vector< double > vl_model, vb_model, vw_model, vdist_model;
		vector< string > vSrcName;
		LoadModel(vsmodelfile[im], vSrcName, vl_model, vb_model, vw_model, vdist_model);

		//for the three scenarios
		for(unsigned int j=0; j<vscenarios.size(); j++){
			const Scenario& s = vscenarios[j];
			stringstream ssout;
			ssout<<"CompoModel/"<<vsfile[im]<<"/"<<vsfile[im]<<"_"<<vscenarios[j].fName<<"_threshold"<<threshold;
			ofstream file_out(ssout.str().c_str());
			for(unsigned int k=0; k<vSrcName.size(); k++){
				double compo_weight = LoadWeight(crpFile,vdist_model[k], masses, threshold, s);
				double tot_weight = compo_weight*vw_model[k];
				file_out<<vSrcName[k]<<" "<<vl_model[k]<<" "<<vb_model[k]<<" "<<tot_weight<<"\n";
			}
			file_out.close();
		}

		//without weight
		stringstream ssout;
		ssout<<"CompoModel/"<<vsfile[im]<<"/"<<vsfile[im]<<"__threshold"<<threshold;
		ofstream file_out(ssout.str().c_str());
		for(unsigned int k=0; k<vSrcName.size(); k++) file_out<<vSrcName[k]<<" "<<vl_model[k]<<" "<<vb_model[k]<<" "<<vw_model[k]<<"\n";
		file_out.close();
	}

	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                         End of the code                                //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////

	cout << endl <<"------ Program Finished Normally: Good Job! ------" << endl;
	gSystem->Exit(0);
	rint->Run(kTRUE);

}
